require "import"
import "android.speech.RecognizerIntent"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "com.androlua.*"
import "android.net.Uri"
import "android.content.Intent"
import "java.util.Locale"

local boobalan = "" -- variable to store the spoken words

-- Function to save recognized words
function saveWords(words)
    boobalan = words
end

-- Function to safely stop the SpeechRecognizer
function safeStop(speechRecognizer)
    if speechRecognizer ~= nil then
        speechRecognizer.stopListening()
        speechRecognizer.destroy()
    end
end

-- Function to start speech recognition
function startListening()
    local verbalizationDetected = false

    -- Create an intent for speech recognition
    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ur-PK")  -- Set to Urdu language
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())

    local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)

    -- Create a listener for the recognition results
    local speechListener = RecognitionListener{
        onResults = function(results)
            local data = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if data and data.size() > 0 then
                verbalizationDetected = true
                local recognizedText = data.get(0)  -- Get the first recognized text

                -- Save the spoken words
                saveWords(recognizedText)

                -- Apply a basic translation or replacement
                local wordChangeTable = {
                    ["میرا پہلا نام"] = "Syed Wajeeh Ul Hassan Naqvi",
                    ["میرا پہلا ایڈریس"] = "Mohallah Mohammed Pura near National Bank, Main Branch, Narowal, Pakistan",
                    ["میرا پہلا موبائل نمبر"] = "+923116674054",
                    ["میرا پہلا جی میل ایڈریس"] = "syedwajeehulhassannaqvi1@gmail.com",
                    ["مثال"] = "example"
                }

                -- Apply word replacements from the table
                for k, v in pairs(wordChangeTable) do
                    recognizedText = recognizedText:gsub(k, v)
                end

                -- Check if translation is needed
                local defaultLanguage = Locale.getDefault().getLanguage()
                local languageCode = "en"  -- Translate to English

                if languageCode then
                    local wordsToTranslate = recognizedText
                    local translatedWords = ""

                    -- Call Google Translate API
                    local url = "https://translate.google.com/m?sl=auto&hl=" .. defaultLanguage .. "&q=" .. Uri.encode(wordsToTranslate)
                    url = url .. "&tl=" .. languageCode

                    Http.get(url, function(status, result)
                        if status == 200 and result then
                            -- Parse translation result
                            translatedWords = result:match('<div class="result%-container">([^<]+)</div>')
                            if translatedWords then
                                -- Output translated text
                                service.insertText(node, translatedWords .. ". \n")
                                this.speak(translatedWords)
                            else
                                service.speak("Translation response was invalid.")
                            end
                        else
                            service.speak("Translation failed. Check your internet connection.")
                        end
                    end)
                else
                    -- Output original recognized text if no translation needed
                    service.insertText(node, recognizedText .. ". \n")
                    this.speak(recognizedText)
                end
            else
                -- No verbalization detected
                service.speak("Sorry! I did not understand this message.")
            end

            -- Safely stop the recognizer
            safeStop(speechRecognizer)
        end,

        -- Handle errors during speech recognition
        onError = function(errorCode)
            local errorMessages = {
                [SpeechRecognizer.ERROR_NO_MATCH] = "Sorry! I did not understand your message.",
                [SpeechRecognizer.ERROR_SPEECH_TIMEOUT] = "Speech timeout.",
                [SpeechRecognizer.ERROR_CLIENT] = "Client error.",
                [SpeechRecognizer.ERROR_NETWORK] = "Network error.",
                [SpeechRecognizer.ERROR_NETWORK_TIMEOUT] = "Network timeout.",
                [SpeechRecognizer.ERROR_SERVER] = "Server error.",
                [SpeechRecognizer.ERROR_AUDIO] = "Audio error.",
                [SpeechRecognizer.ERROR_INSUFFICIENT_PERMISSIONS] = "Insufficient permissions."
            }
            local errorMessage = errorMessages[errorCode] or "An unexpected error occurred."
            service.speak(errorMessage)
            safeStop(speechRecognizer)
        end
    }

    -- Set the listener for the speech recognizer
    speechRecognizer.setRecognitionListener(speechListener)
    speechRecognizer.startListening(recognizerIntent)
end

-- Start listening for speech
startListening()

return true